﻿//------------------------------------------------------------------------------
// <copyright file=”*.cs” company=”EhLib Team”>
//     Copyright (c) 2017 Dmitry V. Bolshakov   
// </copyright>
//------------------------------------------------------------------------------

using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Windows.Forms.VisualStyles;

namespace EhLib
{

  public class DataVertGridCheckBoxRowQueryCheckStateEventArgs : HandledEventArgs
  {
    private object listItem;
    private CheckState checkState;

    public DataVertGridCheckBoxRowQueryCheckStateEventArgs(object listItem)
    {
      this.listItem = listItem;
    }

    public object ListItem
    {
      get { return this.listItem; }
      internal set { this.listItem = value; }
    }

    public CheckState CheckState
    {
      get { return this.checkState; }
      set { this.checkState = value; }
    }

  }

  [DataAxisGridPropBarDesignTimeVisibleAttribute(true)]
  public class DataVertGridCheckBoxRow : DataVertGridMonoCellRow, ICheckBoxDataCellHolder
  {

    #region private consts
    private static readonly object EventKeyQueryCheckboxState = new object();
    #endregion

    #region privates
    #endregion

    public DataVertGridCheckBoxRow()
    {

    }

    #region run-rime properties
    [Browsable(false)]
    public new DataAxisGridCheckBoxDataCell DataCell
    {
      get { return (DataAxisGridCheckBoxDataCell)base.DataCell; }
    }
    #endregion

    #region properties
    [DefaultValue(false)]
    public bool ThreeState
    {
      get { return DataCell.ThreeState; }
      set { DataCell.ThreeState = value; }
    }

    [DefaultValue(null)]
    [TypeConverter(typeof(StringConverter))]
    public object TrueValue
    {
      get
      {
        return DataCell.TrueValue;
      }

      set
      {
        DataCell.TrueValue = value;
      }
    }

    [DefaultValue(null)]
    [TypeConverter(typeof(StringConverter))]
    public object FalseValue
    {
      get
      {
        return DataCell.FalseValue;
      }

      set
      {
        DataCell.FalseValue = value;
      }
    }

    [DefaultValue(null)]
    [TypeConverter(typeof(StringConverter))]
    public object IndeterminateValue
    {
      get
      {
        return DataCell.IndeterminateValue;
      }

      set
      {
        DataCell.IndeterminateValue = value;
      }
    }

    [DefaultValue(FlatStyle.Standard)]
    public FlatStyle FlatStyle
    {
      get
      {
        return DataCell.FlatStyle;
      }

      set
      {
        DataCell.FlatStyle = value;
      }
    }

    #endregion

    #region events
    public event EventHandler<DataGridCheckBoxColumnQueryCheckStateEventArgs> QueryCheckState
    {
      add
      {
        this.Events.AddHandler(EventKeyQueryCheckboxState, value);
      }
      remove
      {
        this.Events.RemoveHandler(EventKeyQueryCheckboxState, value);
      }
    }
    #endregion

    #region methods
    protected override DataAxisGridDataCellMan CreateTemplateCell()
    {
      return new DataAxisGridCheckBoxDataCell();
    }

    protected internal virtual CheckState OnGetCheckState(DataVertGridRow column, object listItem, out bool handled)
    {
      DataVertGridCheckBoxRowQueryCheckStateEventArgs e = null;
      var eh = this.Events[EventKeyQueryCheckboxState] as EventHandler<DataVertGridCheckBoxRowQueryCheckStateEventArgs>;
      if (eh != null /*&& !this.IsDisposed*/)
      {
        e = new DataVertGridCheckBoxRowQueryCheckStateEventArgs(listItem);
        eh(this, e);
      }
      if ((e != null) && e.Handled)
      {
        handled = true;
        return e.CheckState;
      }
      else
      {
        handled = false;
        return CheckState.Indeterminate;
      }
    }

    CheckState ICheckBoxDataCellHolder.OnGetCheckState(PropertyAxisBar propAxisBar, object listItemBar, out bool handled)
    {
      return OnGetCheckState((DataVertGridRow)propAxisBar, listItemBar, out handled);
    }
    #endregion

  }

}
